<?php

namespace DK_GF_SIB_FREE;

use GFForms;
use GFFeedAddOn;
use DK_GF_SIB_FREE\Helpers;

defined( 'ABSPATH' ) || exit;

GFForms::include_feed_addon_framework();

define( 'DKGFSIB_FREE_ADDON_CACHE_BUSTER', 1 );

/**
 * This class is in charge of the GF add-on.
 */
class GF_Addon extends GFFeedAddOn {
	protected $_version                  = DKGFSIB_FREE_VERSION;
	protected $_min_gravityforms_version = '1.9';
	protected $_slug                     = DKGFSIB_FREE_PLUGIN_DIRNAME;
	protected $_path                     = DKGFSIB_FREE_BASENAME;
	protected $_full_path                = DKGFSIB_FREE_DIR . DKGFSIB_FREE_PLUGIN_DIRNAME . '.php';
	protected $_title                    = 'Gravity Forms Sendinblue Add-On';
	protected $_short_title              = 'Sendinblue';
	protected $_multiple_feeds           = false;

	/**
	 * Singleton holder: if available, contains an instance of this class.
	 *
	 * @var GF_Addon|null
	 */
	private static $_instance = null;

	/**
	 * Returns an instance of this class, and stores it in the $_instance property.
	 *
	 * @return GF_Addon $_instance An instance of this class.
	 */
	public static function get_instance() {
		if ( self::$_instance == null ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	// ================================================
	//
	// ###    ####    ###    ###  ##  ##     ##
	// ## ##   ##  ##  ## #  # ##  ##  ####   ##
	// ##   ##  ##  ##  ##  ##  ##  ##  ##  ## ##
	// #######  ##  ##  ##      ##  ##  ##    ###
	// ##   ##  ####    ##      ##  ##  ##     ##
	//
	// ================================================

	/**
	 * Enqueue admin styles.
	 *
	 * @return array
	 */
	public function styles() {
		$styles = array(
			array(
				'handle'  => 'dkgfsib-free-admin-style',
				'src'     => sprintf( '%1$s/assets/css/%2$s', DKGFSIB_FREE_URL, defined( 'WPC_USE_UNMINIFIED_ASSETS' ) && WPC_USE_UNMINIFIED_ASSETS ? 'admin.css' : 'admin.min.css' ),
				'version' => $this->_version . '~' . DKGFSIB_FREE_ADDON_CACHE_BUSTER,
				'enqueue' => array( 'admin_page' => array( 'form_settings', 'plugin_settings' ) ),
			),
		);

		return array_merge( parent::styles(), $styles );
	}

	public function scripts() {
		$scripts = array(
			array(
				'handle'    => 'dkgfsib-free-admin-script',
				'src'       => sprintf( '%1$s/assets/js/%2$s', DKGFSIB_FREE_URL, defined( 'WPC_USE_UNMINIFIED_ASSETS' ) && WPC_USE_UNMINIFIED_ASSETS ? 'admin.js' : 'admin.min.js' ),
				'version'   => $this->_version . '~' . DKGFSIB_FREE_ADDON_CACHE_BUSTER,
				'deps'      => array( 'jquery', 'wp-i18n' ),
				'in_footer' => false,
				'callback'  => array( $this, 'localize_scripts' ),
				'enqueue'   => array( 'admin_page' => array( 'form_settings', 'plugin_settings' ) ),
			),
		);

		return array_merge( parent::scripts(), $scripts );
	}

	/**
	 * The the add-on settings icon.
	 *
	 * @return string
	 */
	public function get_menu_icon() {
		// assets/images/sendinblue-logo.svg
		return '<?xml version="1.0" encoding="UTF-8"?><svg clip-rule="evenodd" fill-rule="evenodd" stroke-linejoin="round" stroke-miterlimit="2" version="1.1" viewBox="0 0 10 10" xml:space="preserve" xmlns="http://www.w3.org/2000/svg" width="20" height="20"><path d="m8.981 7.158c-0.276 0.444-0.727 0.772-1.253 0.912 0.075-0.245 0.113-0.491 0.113-0.748-0-1.252-0.927-2.304-2.168-2.597 0.263-0.234 0.576-0.409 0.927-0.491 0.539-0.14 1.128-0.07 1.604 0.199 1.015 0.561 1.366 1.778 0.777 2.725m-3.984 2.164c-0.552-0-1.078-0.211-1.466-0.562 0.263-0.058 0.513-0.152 0.739-0.28 1.153-0.632 1.666-1.907 1.303-3.065 0.889 0.234 1.541 1.006 1.541 1.907 0 1.111-0.952 2-2.117 2m-3.985-2.164c-0.275-0.445-0.351-0.983-0.213-1.474 0.188 0.176 0.401 0.339 0.627 0.468 0.426 0.234 0.914 0.363 1.415 0.363 0.777-0 1.517-0.293 2.055-0.819 0.226 0.842-0.163 1.754-0.99 2.199-0.325 0.175-0.689 0.269-1.052 0.269-0.764-0.012-1.466-0.386-1.842-1.006m0-4.316c0.276-0.444 0.727-0.772 1.253-0.912-0.075 0.245-0.113 0.491-0.113 0.748 0 1.252 0.928 2.304 2.168 2.597-0.664 0.596-1.692 0.748-2.531 0.292-0.489-0.269-0.839-0.69-0.99-1.204-0.15-0.515-0.075-1.065 0.213-1.521m3.985-2.164c0.551 0 1.077 0.211 1.465 0.562-0.263 0.058-0.513 0.152-0.739 0.28-0.664 0.363-1.127 0.936-1.328 1.626-0.138 0.468-0.125 0.971 0.013 1.439-0.89-0.234-1.541-1.006-1.541-1.907 0.012-1.099 0.964-2 2.13-2m3.984 2.164c0.276 0.456 0.351 0.983 0.213 1.474-0.188-0.187-0.401-0.339-0.627-0.468-0.664-0.363-1.428-0.456-2.167-0.269-0.501 0.129-0.965 0.386-1.316 0.737-0.087-0.328-0.087-0.667 0.013-0.994 0.15-0.515 0.501-0.948 0.99-1.205 0.488-0.269 1.065-0.339 1.603-0.199 0.552 0.14 1.003 0.468 1.291 0.924m0.639-0.339c-0.376-0.62-0.99-1.064-1.729-1.252-0.176-0.046-0.351-0.07-0.526-0.081-0.527-0.726-1.416-1.17-2.356-1.17-0.977-0-1.842 0.468-2.355 1.17-0.94 0.07-1.792 0.561-2.268 1.333-0.376 0.62-0.476 1.333-0.288 2.023 0.05 0.164 0.112 0.316 0.187 0.468-0.413 0.795-0.375 1.731 0.101 2.503 0.488 0.795 1.353 1.263 2.255 1.333 0.526 0.726 1.416 1.17 2.356 1.17 0.977-0 1.841-0.468 2.355-1.17 0.94-0.07 1.792-0.561 2.268-1.333 0.489-0.795 0.489-1.731 0.1-2.503 0.401-0.783 0.363-1.731-0.1-2.491" fill-rule="nonzero"/></svg>';
	}

	/**
	 * Global plugin settings.
	 *
	 * @return array
	 */
	public function plugin_settings_fields() {
		return array(
			array(
				'title'  => esc_html__( 'Sendinblue Add-On Settings', 'addon-gravityforms-sendinblue-free' ),
				'fields' => array(
					array(
						'name'  => 'license-key-field',
						'label' => esc_html__( 'License Activation', 'addon-gravityforms-sendinblue-free' ),
						'type'  => 'wpc_license_field',
					),
					array(
						'name'              => 'sib-api-key',
						/* translators: %1$s: Sendinblue account API URL */
						'tooltip'           => sprintf( esc_attr__( 'Visit the <a href="%1$s" target="_blank">"Settings > SMTP & API"</a> page in your Sendinblue account to generate an API key.', 'addon-gravityforms-sendinblue-free' ), 'https://account.sendinblue.com/advanced/api' ),
						'label'             => esc_html__( 'Sendinblue API Key', 'addon-gravityforms-sendinblue-free' ),
						'type'              => 'text',
						'class'             => 'small',
						'feedback_callback' => array( $this, 'validate_sendinblue_api_key_field' ),
					),
				),
			),
		);
	}

	/**
	 * Output the custom license field/button section.
	 *
	 * @return void
	 */
	public function settings_wpc_license_field() {
		echo '<div class="license-key-container">';

		$this->settings_text(
			array(
				'name'     => 'wpc-sib-license',
				'value'    => '',
				'label'    => esc_html__( 'License Key', 'addon-gravityforms-sendinblue-free' ),
				'disabled' => true,
			)
		);
		/*
		printf(
			'<a target="_blank" href="%1$s" class="primary button large activate">%2$s</a>',
			'https://wpconnect.co/gravity-forms-sendinblue-add-on/',
			esc_html__( 'Upgrade to Pro version', 'addon-gravityforms-sendinblue-free' )
		);*/

		echo '</div>';
	}

	/**
	 * Check/activate the plugin license key.
	 *
	 * @param string $value
	 * @return boolean
	 */
	public function validate_license_key_field( $value ) {
		if ( empty( $value ) ) {
			return false;
		}

		return true;

		// return Helpers\activate_plugin_license_key( $value );
	}

	/**
	 * Check the Sendinblue API key validity.
	 *
	 * @param string $value
	 * @return boolean
	 */
	public function validate_sendinblue_api_key_field( $value ) {
		if ( empty( $value ) ) {
			return false;
		}

		$transient_name = sprintf( '%1$ssib_api_key_%2$s', DKGFSIB_FREE_OPTIONS_PREFIX, substr( md5( sanitize_text_field( $value ) ), 0, 12 ) );
		$result         = get_transient( $transient_name );

		if ( $result === false ) {
			$api    = dkgfsib_free_get_api( $value );
			$result = $api->get_account();

			set_transient( $transient_name, $result, 5 * MINUTE_IN_SECONDS );
		}

		Helpers\process_sendinblue_account_response(
			! is_wp_error( $result ) ? $result : null,
			$result
		);

		return ! is_wp_error( $result );
	}

	/**
	 * Define feeds table columns.
	 *
	 * @return array
	 */
	public function feed_list_columns() {
		return array(
			'name'           => esc_html__( 'Name', 'addon-gravityforms-sendinblue-free' ),
			'lists'          => esc_html__( 'List(s)', 'addon-gravityforms-sendinblue-free' ),
			'has_conditions' => esc_html__( 'Condition(s)', 'addon-gravityforms-sendinblue-free' ),
		);
	}

	/**
	 * Get column: has_conditions.
	 *
	 * @param array $feed
	 * @return void
	 */
	public function get_column_value_has_conditions( $feed ) {
		if ( isset( $feed['meta'], $feed['meta']['feed_condition_conditional_logic'] ) && (int) $feed['meta']['feed_condition_conditional_logic'] === 1 ) {
			echo '<span class="icon-yes dashicons dashicons-yes"></span>';
		} else {
			echo '<span class="icon-no dashicons dashicons-no-alt"></span>';
		}
	}

	/**
	 * Can we create a feed?
	 *
	 * @return boolean
	 */
	public function can_create_feed() {
		return dkgfsib_free_api_key_is_valid();
	}

	/**
	 * Per-form settings.
	 *
	 * @return array
	 */
	public function feed_settings_fields() {
		if ( ! dkgfsib_free_api_key_is_valid() ) {
			$message = sprintf(
				/* translators: %1$s: Gravity Form Sendinblue settings page URL */
				__( 'Please visit the <a href="%1$s">plugin settings page</a> and enter a valid Sendinblue API key.', 'addon-gravityforms-sendinblue-free' ),
				admin_url( 'admin.php?page=gf_settings&subview=dk-gravityforms-sendinblue' )
			);

			$fields = array(
				array(
					'name'  => 'sib-api-key-invalid',
					'label' => esc_html__( 'API key missing or invalid', 'addon-gravityforms-sendinblue-free' ),
					'type'  => 'html',
					'html'  => sprintf( '<p>%1$s</p>', $message ),
				),
			);
		} else {
			$attributes = Helpers\get_sendinblue_attributes();
			$lists      = Helpers\get_sendinblue_contact_lists();

			$is_enabled_dependency = array(
				'live'     => true,
				'operator' => 'ALL',
				'fields'   => array(
					array(
						'field'  => 'is-enabled',
						'values' => array( '1', true ),
					),
				),
			);
			$attributes            = is_array( $attributes ) ? $attributes : array();

            $attributes_choices = array_map(
				function( $attribute ) {
                    // Allow only boolean (opt-in like field)
                    if ($attribute->type === 'boolean') {
                        return array(
                            'label' => sanitize_text_field( $attribute->name ),
                            'value' => sanitize_text_field( $attribute->name ),
                        );
                    }
					return array(
						'label' => sanitize_text_field( $attribute->name ) . ' ' . esc_html__( '(Pro version)', 'addon-gravityforms-sendinblue-free' ),
						'value' => '***disabled***', // We can't add disable attribute here so we change the value and make the option disabled in JS (@See admin.js)
					);
				},
                $attributes
			);

            uasort($attributes_choices, function ($a, $b) {
                return $a['value'] !== '***disabled***' ? -1 : 1;
            });

			array_unshift(
				$attributes_choices,
				array(
					'label' => esc_html__( 'E-mail address', 'addon-gravityforms-sendinblue-free' ),
					'value' => 'EMAIL',
				)
			);

			$fields = array(
				array(
					'name'  => 'is-enabled',
					'label' => esc_html__( 'Enabled', 'addon-gravityforms-sendinblue-free' ),
					'type'  => 'toggle',
				),
				array(
					'name'       => 'name',
					'label'      => esc_html__( 'Feed name', 'addon-gravityforms-sendinblue-free' ),
					'type'       => 'text',
					'required'   => true,
					'class'      => 'medium',
					'tooltip'    => esc_html__( 'Enter a feed name to uniquely identify it.', 'addon-gravityforms-sendinblue-free' ),
					'dependency' => $is_enabled_dependency,
				),
				array(
					'name'              => 'mapped-fields',
					'label'             => esc_html__( 'Fields mapping', 'addon-gravityforms-sendinblue-free' ),
					'tooltip'           => esc_html__( 'Add and select the Sendinblue attributes for which to send data, then choose the form fields that match.', 'addon-gravityforms-sendinblue-free' ),
					'type'              => 'dynamic_field_map',
					'required'          => true,
					'enable_custom_key' => false,
					'value_field'       => array( 'title' => esc_html__( 'Form fields', 'addon-gravityforms-sendinblue-free' ) ),
					'key_field'         => array(
						'title'   => esc_html__( 'Sendinblue attributes', 'addon-gravityforms-sendinblue-free' ),
						'choices' => $attributes_choices,
					),
					'dependency'        => $is_enabled_dependency,
					'fields'            => array(
						array(
							'name'  => 'sib-free-mapping-notice',
							'label' => '',
							'type'  => 'html',
							'html'  => '<p>' .
								sprintf(
									/* translators: %1$s Pro version link */
									esc_html__( 'Free version only includes E-mail and Consent fields ("OPT_IN" boolean in Sendinblue), while the %1$s is unlimited.', 'addon-gravityforms-sendinblue-free' ) . '</p>',
									sprintf(
										'<a href="%s" target="_blank">%s</a>',
										'https://wpconnect.co/gravity-forms-sendinblue-add-on',
										esc_html__( 'Pro version', 'addon-gravityforms-sendinblue-free' )
									)
								),

						),
					),
				),

			);

			if ( is_array( $lists ) && ! empty( $lists ) ) {
				$lists_field = array(
					'name'       => 'add-to-list',
					'label'      => esc_html__( 'Add to list', 'addon-gravityforms-sendinblue-free' ),
					'tooltip'    => esc_html__( 'All your Sendinblue lists are displayed below. You can choose which one(s) to save each new contact to.', 'addon-gravityforms-sendinblue-free' ),
					'type'       => 'radio',
					'horizontal' => true,
					'choices'    => array_merge(
						array(
							array(
								'label' => esc_html__( 'None', 'addon-gravityforms-sendinblue-free' ),
								'name'  => 'add-to-list-none',
								'value' => 0,
							),
						),
						array_map(
							function( $list ) {
								return array(
									'label' => sanitize_text_field( $list->name ),
									'name'  => sprintf( 'add-to-list', sanitize_text_field( $list->id ) ),
									'value' => sanitize_text_field( $list->id ),
								);
							},
							$lists
						)
					),
					'dependency' => $is_enabled_dependency,
				);

				if ( count( $lists ) > 1 ) {
					$lists_field['fields'] = array(
						array(
							'name'  => 'sib-free-lists-notice',
							'label' => '',
							'type'  => 'html',
							'html'  => '<p>' .
								sprintf(
									/* translators: %1$s Pro version link */
									esc_html__( 'Free version only allows you to select one list at a time, while the %1$s is unlimited.', 'addon-gravityforms-sendinblue-free' ) . '</p>',
									sprintf(
										'<a href="%s" target="_blank">%s</a>',
										'https://wpconnect.co/gravity-forms-sendinblue-add-on',
										esc_html__( 'Pro version', 'addon-gravityforms-sendinblue-free' )
									)
								),

						),
					);
				}

				$fields[] = $lists_field;

				$fields[] = array(
					'name'       => 'use-double-optin',
					'label'      => esc_html__( 'Double opt-in (Pro version)', 'addon-gravityforms-sendinblue-free' ),
					'type'       => 'checkbox',
					'disabled'   => true,
					'choices'    => array(
						array(
							'label' => esc_html__( 'Enable double opt-in', 'addon-gravityforms-sendinblue-free' ) . ' ' . esc_html__( '(Pro version)', 'addon-gravityforms-sendinblue-free' ),
							'name'  => 'double-optin-enabled',
						),
					),
					'tooltip'    => esc_html__( 'By enabling this, Sendinblue will send a confirmation email to the user before adding them to your list(s).', 'addon-gravityforms-sendinblue-free' ),
					'dependency' => $is_enabled_dependency,
				);
			}
		}

		return array(
			array(
				'title'  => esc_html__( 'Integration with Sendinblue', 'addon-gravityforms-sendinblue-free' ),
				'fields' => $fields,
				'class'  => 'wpco-sib-integration-settings',
				'id'     => 'wpco-sib-integration-settings',
			),
			array(
				'title'  => esc_html__( 'Conditional logic', 'addon-gravityforms-sendinblue-free' ),
				'class'  => 'wpco-sib-conditional-logic',
				'id'     => 'wpco-sib-conditional-logic',
				'fields' => array(
					array(
						'type'           => 'feed_condition',
						'name'           => 'feed-condition',
						'label'          => esc_html__( 'Conditions', 'addon-gravityforms-sendinblue-free' ),
						'checkbox_label' => esc_html__( 'Enable conditional processing', 'addon-gravityforms-sendinblue-free' ),
						'instructions'   => esc_html__( 'Create a contact in Sendinblue only if', 'addon-gravityforms-sendinblue-free' ),
					),
				),
			),
		);
	}

	// =================================================
	//
	// #####  ##     ##  ######  #####    ##    ##
	// ##     ####   ##    ##    ##  ##    ##  ##
	// #####  ##  ## ##    ##    #####      ####
	// ##     ##    ###    ##    ##  ##      ##
	// #####  ##     ##    ##    ##   ##     ##
	//
	// =================================================

	/**
	 * Initiate processing the feed.
	 *
	 * @since  1.0
	 *
	 * @param array $feed  The Feed object to be processed.
	 * @param array $entry The Entry object currently being processed.
	 * @param array $form  The Form object currently being processed.
	 */
	public function process_feed( $feed, $entry, $form ) {
		$settings = $feed['meta'];

		if (
			! (bool) $feed['is_active']
			|| ! isset( $settings['mapped-fields'] )
			|| ! (bool) $settings['is-enabled']
		) {
			return;
		}

		$mapped_fields = array_map(
			function( $conf ) use ( $entry, $form ) {
				$value = isset( $entry[ $conf['value'] ] ) ? sanitize_text_field( $entry[ $conf['value'] ] ) : null;
				$key   = ( $conf['key'] === 'gf_custom' ) ? sanitize_text_field( $conf['custom_key'] ) : sanitize_text_field( $conf['key'] );

				if ( $conf['value'] === 'form_title' ) {
					$value = sanitize_text_field( $form['title'] );
				}

				return array(
					'key'   => $key,
					'value' => $value,
				);
			},
			$settings['mapped-fields']
		);

		$mapped_fields          = $this->get_dynamic_field_map_fields( $feed, 'mapped-fields' );
		$enhanced_mapped_fields = array();

		foreach ( $mapped_fields as $key => $field_id ) {
			if ( rgblank( $field_id ) ) {
				continue;
			}
            $field_value = $this->get_field_value( $form, $entry, $field_id );
            $field = \GFFormsModel::get_field( $form, $field_id );

            if ( is_object( $field ) ) {
                $input_type = $field->get_input_type();
                if ($input_type === 'consent' && in_array($field_value, array(esc_html__( 'Not Checked', 'gravityforms' ), esc_html__( 'Checked', 'gravityforms' )))) {
                    $field_value = $field_value === esc_html__( 'Checked', 'gravityforms' );
                }
            }

			$enhanced_mapped_fields[ $key ] = (object) array(
				'key'   => $key,
				'value' => $field_value,
			);
		}

		$list_id = ( isset( $settings['add-to-list'] ) ? (int) $settings['add-to-list'] : -1 );
		if ( $list_id > 0 ) {
			$lists = array( $list_id );
		} else {
			$lists = array();
		}

		// Filter variables for external modifications.
		$mapped_fields = apply_filters( 'dk-gf-sib-free/entry/mapped-fields', $enhanced_mapped_fields, $feed, $entry, $form, $settings, $this );
		$lists         = apply_filters( 'dk-gf-sib-free/entry/lists', $lists, $feed, $entry, $form, $settings, $this );

		if ( ! apply_filters( 'dk-gf-sib-free/disable-form-entry-processing', false, $feed, $entry, $form, $settings, $this ) ) {
			do_action(
				'dk-gf-sib-free/process-form-entry',
				$mapped_fields,
				$lists,
				false,
				null,
				null,
				$feed,
				$entry,
				$form,
				$settings,
				$this
			);
		}
	}

	/**
	 * Get all feeds API responses for a specific entry.
	 *
	 * @param integer $entry_id
	 * @return array
	 */
	public function get_entry_addon_metadata( $entry_id ) {
		$metas = array();

		$feeds = $this->get_feeds_by_entry( $entry_id );

		if ( ! is_array( $feeds ) ) {
			return $metas;
		}

		foreach ( $this->get_feeds_by_entry( $entry_id ) as $feed_id ) {
			$metas[ $feed_id ] = gform_get_meta( $entry_id, sprintf( 'sib_contact_api_response:%1$d', $feed_id ) );
		}

		return $metas;
	}
}
